#include "portfile.h"
#include "gen.h"
/* This is a collection of functions that allows you to
write binary files or binary network data in a machine
independant byte order. The algorithm is simply to
extract the most significant byte and put it first
*/

/* There is no need to define LITTLE_ENDIAN whether or
not your machine is is little endian.
By default files will be stored in big endian format
whether or not your machine is big endian, thus making
files portable. If you have a strong preference for
storing your files in little endian format then you can
define LITTLE_ENDIAN, but your files won't be compatible
with any machine (whether litle endian or big endian)
that didn't have LITTLE_ENDIAN defined.
*/

/* NOTE!! The algorithms in here RELY on Buffer type being
unsigned char !!! */


/*
#define LITTLE_ENDIAN
*/
#ifdef LITTLE_ENDIAN
#define PORT_MS_US 1 /* most significant index for unsigned short */
#define PORT_LS_US 0 /* least significant index for unsigned short */

#define PORT_MMS_UL 3 /* most most significant index for unsigned long */
#define PORT_MS_UL  2 /* most significant index for unsigned long */
#define PORT_LS_UL  1 /* least significant index for unsigned long */
#define PORT_LLS_UL 0 /* least least significant index for unsigned long */
#else 
#define PORT_MS_US 0 /* most significant index for unsigned short */
#define PORT_LS_US 1 /* least significant index for unsigned short */

#define PORT_MMS_UL 0 /* most most significant index for unsigned long */
#define PORT_MS_UL  1 /* most significant index for unsigned long */
#define PORT_LS_UL  2 /* least significant index for unsigned long */
#define PORT_LLS_UL 3 /* least least significant index for unsigned long */
#endif

void portUint16(Buffer *buf, Uint16 *ptr, Bool un) 
{ 
#	ifdef DEB1
	errfmt("portUint16: beg\n");
#	endif
	if (un)
		{
		*ptr = 0;
		*ptr |= ((Uint16)buf[PORT_LS_US]);
		*ptr |= ((Uint16)buf[PORT_MS_US]) << 8;
		}
	else
		{
		buf[PORT_LS_US] =  (*ptr & 0xff); 
		buf[PORT_MS_US] = (*ptr & 0xff00) >> 8; 
		}
#	ifdef DEB1
	errfmt("portUint16: end\n");
#	endif
}

void portUint32(Buffer *buf, Uint32 *ptr, Bool un) 
	{ 
#	ifdef DEB1
	errfmt("portUint32: beg un? %d\n", un);
#	endif
	if (un)
		{
#	ifdef DEB1
	errfmt("portUint32:1 beg un? %d\n", un);
#	endif
		*ptr = 0L;
#	ifdef DEB1
	errfmt("portUint32:2 beg un? %d\n", un);
#	endif
		*ptr |= ((Uint32)buf[PORT_LLS_UL]);
#	ifdef DEB1
	errfmt("portUint32:3 beg un? %d\n", un);
#	endif
		*ptr |= ((Uint32)buf[PORT_LS_UL]) << 8;
#	ifdef DEB1
	errfmt("portUint32:4 beg un? %d\n", un);
#	endif
		*ptr |= ((Uint32)buf[PORT_MS_UL]) << 16;
#	ifdef DEB1
	errfmt("portUint32:5 beg un? %d\n", un);
#	endif
		*ptr |= ((Uint32)buf[PORT_MMS_UL]) << 24;
#	ifdef DEB1
	errfmt("portUint32:6 beg un? %d\n", un);
#	endif
		}
	else
		{
		buf[PORT_LLS_UL] = (*ptr & 0xffL); 
		buf[PORT_LS_UL] = (*ptr & 0xff00L) >> 8; 
		buf[PORT_MS_UL] = (*ptr & 0xff0000L) >> 16; 
		buf[PORT_MMS_UL] = (*ptr & 0xff000000L) >> 24; 
		}
#	ifdef DEB1
	errfmt("portUint32: end\n");
#	endif
	}

void portText(Buffer *buf, Text *ptr, Bool un)
	{
	if (un)
		strcpy(ptr, buf);
	else
		strcpy(buf, ptr);
	}

void portBuffer(Buffer *buf, Buffer *ptr, Bool un, size_t size)
	{
	if (un)
		memcpy(ptr, buf, size);
	else
		memcpy(buf, ptr, size);
	}
