#include <stdio.h>
#include <malloc.h>
#include "vector.h"
#include "xalloc.h"

#define VEC_INCREMENT 1024
#define MAX_LINE 1024

static void ** getItems(char * filename, int * size);

void newVector(Vector * vec, int size)
	{
	register void ** ptr;
	register void ** end;

	vec->limit = vec->size = size;
	vec->vecInc = 50;
	if (size == 0)
		vec->items = NULL;
	else
		ptr = vec->items = (void **)malloc(sizeof(void *) * size);

	end = ptr + size;
	while (ptr < end)
		*ptr++ = NULL;
	}

void freeVector(Vector * vec)
	{
	free(vec->items);
	free(vec);
	}

void resizeVec(Vector * vec, int size)
	{
	if (vec->limit < size)
		{
		int limit = size + vec->vecInc;
		vec->items = (void **)safe_realloc(vec->items, sizeof(void *) * limit);
		while (vec->limit < limit)
			vec->items[vec->limit++] = NULL;
		}
	vec->size = size;
	}

void insertVector(Vector * vec, int pos, void * obj)
	{
	int size = vecSize(vec); /* use temp var because vecSize() changes */
	/* assert(pos <= vecSize(vec)) This should be put back in */
	resizeVec(vec, size + 1);
	memmove(vec->items + pos, 
			vec->items + pos + 1, 
			size - pos);
	vec->items[pos] = obj;
	}

void getVectorFile(Vector * vec, char * filename)
	{
	void ** items;
	int size;

	items = getItems(filename, &size);
	newVector(vec, size);
	vec->items = items;
	}

static void ** getItems(char * filename, int * size)
	{
	void ** rtn = NULL;
	FILE * file;
	char buf[MAX_LINE];
	int num = 0;
	char * verName;
	int vsize = 0, popul = 0;
	
	rtn = (void **)malloc((vsize + 1) * sizeof (char *));
	/* in case the file is empty! */
	if ((file = fopen(filename, "r")) != NULL)
		{
		while (fgets(buf, sizeof(buf), file) != NULL)
			{
			buf[strlen(buf)-1] = '\0';
			if (popul == vsize)
				{
				vsize += VEC_INCREMENT;
				rtn = (void **)safe_realloc(rtn, (vsize + 1) * sizeof (char *));
				}
			/* use indexes with realloc because it's dangerous to fiddle
			   with pointers. realloc could move the memory! */
			rtn[popul] = (void *)strdup(buf);
			popul++;
			}
		}
	rtn[popul] = NULL;
	fclose(file);
	if (size != NULL)
		*size = popul;
	return rtn;
	}

void freevector(void ** vec)
	{
	void ** ptr;
	
	ptr = vec;
	while (*ptr != NULL)
		free(*ptr++);
	free(vec);
	}
