#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <Xm/PushB.h>
#include <Xm/Text.h>
#include <Xm/Command.h>
#include <Xm/FileSB.h>
#include <Xm/MessageB.h>
#include <Xm/DragDrop.h>
#include <Xm/BulletinB.h> 
#include <Xm/AtomMgr.h> 
#include <Xm/Label.h>
#include <X11/Xlib.h>
#include <X11/cursorfont.h>
#include "motif.h"

#define MAX_ARGS 20

#define declareCB(name) void name(Widget w, XtPointer client_data, XmAnyCallbackStruct *cbs);
#define declareCBstru(name,stru) void name(Widget w, XtPointer client_data, stru cbs);
#define declareCBcli(name,cli) void name(Widget w, cli client_data,  XmAnyCallbackStruct *cbs);

#define defineRegistration(func, wid) \
void func(Widget w, \
		  XtPointer client_data, \
		  XmAnyCallbackStruct *cbs) \
{ \
	wid = w; \
}


declareCB(registerTextWindowCB);
declareCB(registerOpenMenuCB);
declareCB(registerSearchCommandCB);
declareCB(registerRangeCommandCB);
declareCB(registerStatusLineCB);
declareCB(registerBookmarkStatusLineCB);
declareCB(registerGotoDialogCB);
declareCB(registerBookSelectCB);
declareCB(registerGotoCommandCB);
declareCB(registerBookmarkListCB);
declareCB(registerMenu_BookmarkCB);
declareCB(registerBookmarkNameEntryCB);
declareCB(registerBookmarkVerseEntryCB);
declareCB(registerNextSearchButtonCB);
declareCB(registerPrevSearchButtonCB);
declareCB(registerFirstSearchButtonCB);
declareCB(registerLastSearchButtonCB);
declareCBcli(openVersionCB, char *);
declareCBstru(bookOkCB, XmSelectionBoxCallbackStruct *);
declareCBstru(bookApplyCB, XmSelectionBoxCallbackStruct *);
declareCBstru(bookCancelCB, XmSelectionBoxCallbackStruct *);
declareCBstru(exitCB, XmSelectionBoxCallbackStruct *);
declareCB(newCB);
declareCB(saveAsCB);
declareCB(printerCB);
declareCB(gotoCB);
declareCB(bookListCB);
declareCB(bookmarkCB);
declareCB(searchCB);
declareCB(registerSearchWinCB);
declareCB(search_nextCB);
declareCB(search_prevCB);
declareCB(search_firstCB);
declareCB(search_lastCB);
declareCB(search_cancelCB);
declareCB(search_helpCB);
declareCB(goto_okCB);
declareCB(goto_cancelCB);
declareCB(goto_helpCB);
declareCB(bookmark_openCB);
declareCB(bookmark_loadCB);
declareCB(bookmark_saveCB);
declareCB(bookmark_saveasCB);
declareCB(bookmark_clearCB);
declareCBstru(bookmark_gotoCB, XmListCallbackStruct *);
declareCB(bookmark_cancelCB);
declareCB(bookmark_popdownCB);
declareCB(bookmark_printCB);
declareCB(bookmark_addCB);
declareCB(bookmark_updateCB);
declareCB(bookmark_deleteCB);
declareCBstru(bookmark_selectCB, XmListCallbackStruct *);
declareCBstru(bookmark_positionCB, XmListCallbackStruct *);

#define regMem(name) { #name, (caddr_t)name }

static MRMRegisterArg register_vector[] =
	{
	regMem(registerTextWindowCB),
	regMem(registerOpenMenuCB),
	regMem(registerStatusLineCB),
	regMem(registerBookmarkStatusLineCB),
	regMem(registerBookSelectCB),
	regMem(registerGotoDialogCB),
	regMem(registerSearchCommandCB),
	regMem(registerRangeCommandCB),
	regMem(registerGotoCommandCB),
	regMem(registerBookmarkListCB),
	regMem(registerMenu_BookmarkCB),
	regMem(registerBookmarkNameEntryCB),
	regMem(registerBookmarkVerseEntryCB),
	regMem(registerNextSearchButtonCB),
	regMem(registerPrevSearchButtonCB),
	regMem(registerFirstSearchButtonCB),
	regMem(registerLastSearchButtonCB),
	regMem(registerMenu_BookmarkCB),
	regMem(bookOkCB),
	regMem(bookApplyCB),
	regMem(bookCancelCB),
	regMem(exitCB),
	regMem(newCB),
	regMem(saveAsCB),
	regMem(printerCB),
	regMem(gotoCB),
	regMem(bookListCB),
	regMem(bookmarkCB),
	regMem(searchCB),
	regMem(registerSearchWinCB),
	regMem(search_nextCB),
	regMem(search_prevCB),
	regMem(search_firstCB),
	regMem(search_lastCB),
	regMem(search_cancelCB),
	regMem(search_helpCB),
	regMem(goto_okCB),
	regMem(goto_cancelCB),
	regMem(goto_helpCB),
	regMem(bookmark_openCB),
	regMem(bookmark_loadCB),
	regMem(bookmark_saveCB),
	regMem(bookmark_saveasCB),
	regMem(bookmark_clearCB),
	regMem(bookmark_gotoCB),
	regMem(bookmark_cancelCB),
	regMem(bookmark_popdownCB),
	regMem(bookmark_printCB),
	regMem(bookmark_addCB),
	regMem(bookmark_updateCB),
	regMem(bookmark_deleteCB),
	regMem(bookmark_selectCB),
	regMem(bookmark_positionCB),
	};

Widget toplevel = NULL;

typedef FD WinFd;

typedef struct class
	{
	Version * version;
	Widget mainWin;
	Widget textWidget;
	Widget statusLineWidget;
	Widget openWidget;
	Widget menuBookmarkWidget;
	} MainWin;

#define MAX_MAINWIN 4
MainWin  * mainWins[MAX_MAINWIN];
MainWin * mxwin;

typedef struct
	{
	Widget searchMainWin;
	Widget searchFor;
	Widget range;
	Widget next;
	Widget prev;
	Widget first;
	Widget last;
	Widget drag;
	MainWin * mainWin;
	} SearchWin;

#define MAX_SEARCHWIN 4
SearchWin  * searchWins[MAX_SEARCHWIN];
SearchWin * mxsearch;

typedef struct
	{
	Widget bookmarkMainWin;
	Widget bookmarkStatusLineWidget;
	Widget bookmarkNameEntryWidget;
	Widget bookmarkListWidget;
	Widget bookmarkVerseEntryWidget;
	Widget bookmarkFileWidget;
	BookmarkNum bookmarkLastSelected;
	MainWin * mainWin;
	} BookmarkWin;

#define MAX_BOOKMARKWIN 4
BookmarkWin  * bookmarkWins[MAX_BOOKMARKWIN];
BookmarkWin * mxbookmark;

typedef struct
	{
	Widget gotoMainWin;
	Widget gotoCommandWidget;
	MainWin * mainWin;
	} GotoWin;

#define MAX_GOTOWIN 4
GotoWin  * gotoWins[MAX_GOTOWIN];
GotoWin * mxgoto;

typedef struct
	{
	Widget bookMainWin;
	Widget bookSelectWidget;
	MainWin * mainWin;
	} BookWin;

#define MAX_BOOKWIN 4
BookWin  * bookWins[MAX_BOOKWIN];
BookWin * mxbook;

Atom     COMPOUND_TEXT;

static Cursor watchCursor = None;
static Cursor stdCursor = None;

defineRegistration(registerTextWindowCB, mxwin->textWidget);
defineRegistration(registerStatusLineCB, mxwin->statusLineWidget);
defineRegistration(registerBookmarkStatusLineCB, mxbookmark->bookmarkStatusLineWidget);
defineRegistration(registerSearchCommandCB, mxsearch->searchFor);
defineRegistration(registerRangeCommandCB, mxsearch->range);
defineRegistration(registerBookSelectCB, mxbook->bookSelectWidget);
defineRegistration(registerSearchWinCB, mxsearch->searchMainWin);
defineRegistration(registerGotoDialogCB, mxgoto->gotoMainWin);
defineRegistration(registerGotoCommandCB, mxgoto->gotoCommandWidget);
defineRegistration(registerBookmarkListCB, mxbookmark->bookmarkListWidget);
defineRegistration(registerMenu_BookmarkCB, mxwin->menuBookmarkWidget);
defineRegistration(registerBookmarkNameEntryCB, mxbookmark->bookmarkNameEntryWidget);
defineRegistration(registerBookmarkVerseEntryCB, mxbookmark->bookmarkVerseEntryWidget);
defineRegistration(registerNextSearchButtonCB, mxsearch->next);
defineRegistration(registerPrevSearchButtonCB, mxsearch->prev);
defineRegistration(registerFirstSearchButtonCB, mxsearch->first);
defineRegistration(registerLastSearchButtonCB, mxsearch->last);

#define register_vector_length \
((sizeof register_vector) / (sizeof register_vector[0]))
MrmHierarchy hier;

/* declarations */
void checkShowVerse(MainWin * win, AbsVerse ver, Bool nBook);
void showBookmarks(Widget wList, BookmarkList * blist);
void populateSelectionFromVec(Widget w, char ** vec, int num);
/*              */

MainWin * winOf(Widget w, void ** list, int num, Widget (* get)(void *))
	{
	WinFd c;

	do
		{
		for (c = 0; c < num; c++)
			if (list[c] != NULL && get(list[c]) == w)
				return list[c];
		w = XtParent(w);
		} while (w != NULL);
	return NULL;
	}

Widget mainGet(MainWin * win)
	{
	return win->mainWin;
	}

MainWin * winOfMain(Widget w)
	{
	return winOf(w, (void **)mainWins, MAX_MAINWIN, (Widget (*)(void *))mainGet);
	}

Widget searchGet(SearchWin * win)
	{
	return win->searchMainWin;
	}

SearchWin * winOfSearch(Widget w)
	{
	return (SearchWin *)winOf(w, (void **)searchWins, MAX_SEARCHWIN, (Widget (*)(void *))searchGet);
	}
Widget bookmarkGet(BookmarkWin * win)
	{
	return win->bookmarkMainWin;
	}

BookmarkWin * winOfBookmark(Widget w)
	{
	return (BookmarkWin *)winOf(w, (void **)bookmarkWins, MAX_BOOKMARKWIN, (Widget (*)(void *))bookmarkGet);
	}

Widget gotoGet(GotoWin * win)
	{
	return win->gotoMainWin;
	}

GotoWin * winOfGoto(Widget w)
	{
	return (GotoWin *)winOf(w, (void **)gotoWins, MAX_GOTOWIN, (Widget (*)(void *))gotoGet);
	}

Widget bookGet(BookWin * win)
	{
	return win->bookMainWin;
	}

BookWin * winOfBook(Widget w)
	{
	return (BookWin *)winOf(w, (void **)bookWins, MAX_BOOKWIN, (Widget (*)(void *))bookGet);
	}

static printmessage(char * mesg, Widget w)
	{
		if (w == NULL)
			fprintf(stderr, "%s\n", mesg);
		else
			{
			XtVaSetValues(w, XmNvalue, mesg, NULL);
			XFlush(XtDisplay(w));
			}
	}

static void domessage(char * mesg, Widget w)
	{
	if (mesg == NULL)
		printmessage("", w);
	else
		printmessage(mesg, w);
	}

static void message(MainWin * win, char * mesg)
	{
	domessage(mesg, win->statusLineWidget);
	}

static void bookmarkMessage(BookmarkWin * win, char * mesg)
	{
	domessage(mesg, win->bookmarkStatusLineWidget);
	}

void setTitle(Widget w, char * title)
	{
	XtVaSetValues(XtParent(w), XmNtitle, title,
				  NULL);
	}

void destroyCB(Widget w,
			   XtPointer client_data,
			   XmAnyCallbackStruct *cbs)
	{
	XtDestroyWidget(w);
	}

void errorDialog(char * mesg)
	{
	XmString cs;
	Arg args[10];
	int i = 0;
	Widget dialog, cb;
	
	cs = XmStringCreateSimple(mesg);
	XtSetArg(args[i], XmNmessageString, cs); i++;
	XtSetArg(args[i], XmNdeleteResponse, XmDESTROY); i++;
	dialog = XmCreateErrorDialog(toplevel, "error", args, i);
	XmStringFree(cs);
	XtVaGetValues(dialog, XmNcancelButton, &cb, NULL);
	XtDestroyWidget(cb);
	XtAddCallback(dialog, XmNokCallback, (XtCallbackProc)destroyCB, NULL);
	XtManageChild(dialog);
	}

void showBook(MainWin * win)
	{
	char title[200];

	XtVaSetValues(win->textWidget, XmNvalue, win->version->cBook->data, NULL);
	cpystr(title, "(1) ", win->version->title, " - ", win->version->cBook->title, NULL);
	setTitle(win->mainWin, title);
	}

int runmotif(MotifContext *mcon)
	{
	XtAppMainLoop(mcon->appContext);
	return 0;
	}

String fallbacks[] = 
	{
	"*bib*viewer*columns: 80",
	"*bib*viewer*rows: 20",
	NULL
	};

static busy(Widget w)
	{
	XDefineCursor(XtDisplay(w), XtWindow(w), watchCursor);
	XFlush(XtDisplay(w));
	}

static notbusy(Widget w)
	{
	XDefineCursor(XtDisplay(w), XtWindow(w), stdCursor);
	}

static setCursor(Widget w)
	{
	notbusy(w);
	}

extern Widget doFetch( MrmHierarchy hierarchy_id , String index , Widget parent , Widget *w_return , MrmType *class_return )
	{
	Widget rtn;
	rtn = XtCreatePopupShell(index, topLevelShellWidgetClass, parent, NULL, 0); 
 	if (MrmFetchWidget(hierarchy_id, index, rtn, w_return, class_return) != MrmSUCCESS)
		errorDialog("cannot fetch");
	if (*w_return != NULL)
		{
/*
		if (watchCursor == None)
			watchCursor = XCreateFontCursor(XtDisplay(toplevel), XC_watch);
		if (stdCursor == None)
			stdCursor = XCreateFontCursor(XtDisplay(toplevel), XC_top_left_arrow);
		setCursor(toplevel); */
		XtManageChild(*w_return);
		XtPopup(rtn, XtGrabNone);
		}
	else
		errorDialog("Cannot fetch widget");
	return rtn;
	}

MainWin * newMainWin()
	{
	MainWin * rtn;
	MrmType aType;
	int count;
	WinFd winFd;
	Arg args[MAX_ARGS];
	Atom importList[1];
	int n;

	if ((winFd = getfd(mainWins, MAX_MAINWIN)) < 0)
		errorDialog("Too Many Windows");
	else
		{
		mxwin = rtn = (MainWin *)malloc(sizeof (MainWin));
		memset(rtn, '\0', sizeof(*rtn));
		doFetch(hier, "bib", toplevel, &rtn->mainWin, &aType);
		mainWins[winFd] = rtn;

		importList[0] = COMPOUND_TEXT;
		n = 0;
		XtSetArg(args[n], XmNimportTargets, importList); n++;
		XtSetArg(args[n], XmNnumImportTargets, 1); n++;
		XtSetArg(args[n], XmNdropSiteOperations, XmDROP_COPY); n++;
		XtSetArg(args[n], XmNdropProc, handleDrop); n++;
		/* the last text window that we created */
		XmDropSiteUpdate(rtn->textWidget, args, n);
		}

	return rtn;
	}
XtTranslations          parsed_xlations;

SearchWin * newSearchWin(MainWin * win)
	{
	SearchWin * rtn;
	MrmType aType;
	int count;
	WinFd winFd;
	Arg args[MAX_ARGS];
	Atom importList[1];
	int n;
   
	if ((winFd = getfd(searchWins, MAX_SEARCHWIN)) < 0)
		errorDialog("Too Many Windows");
	else
		{
		mxsearch = rtn = (SearchWin *)malloc(sizeof (SearchWin));
		memset(rtn, '\0', sizeof(*rtn));
		rtn->mainWin = win;
		doFetch(hier, "search_dialog", toplevel, &rtn->searchMainWin, &aType);
		searchWins[winFd] = rtn;

		XtOverrideTranslations(rtn->next, parsed_xlations);
		XtOverrideTranslations(rtn->prev, parsed_xlations);
		XtOverrideTranslations(rtn->first, parsed_xlations);
		XtOverrideTranslations(rtn->last, parsed_xlations);
/*
		XtVaGetValues(rtn->next, XmNtranslations, parsed_xlations, NULL);
		XtVaGetValues(rtn->prev, XmNtranslations, parsed_xlations, NULL);
		XtVaGetValues(rtn->first, XmNtranslations, parsed_xlations, NULL);
		XtVaGetValues(rtn->last, XmNtranslations, parsed_xlations, NULL);
		*/	
#ifdef NEVER
		importList[0] = COMPOUND_TEXT;
		n = 0;
		XtSetArg(args[n], XmNimportTargets, importList); n++;
		XtSetArg(args[n], XmNnumImportTargets, 1); n++;
		XtSetArg(args[n], XmNdropSiteOperations, XmDROP_COPY); n++;
		XtSetArg(args[n], XmNdropProc, handleDrop); n++;
		/* the last text window that we created */
		XmDropSiteUpdate(winOfMain(rtn->searchMainWin)->textWidget, args, n);
#endif
		}

	return rtn;
	}

GotoWin * newGotoWin(MainWin * win)
	{
	GotoWin * rtn;
	MrmType aType;
	int count;
	WinFd winFd;
	Arg args[MAX_ARGS];
	Atom importList[1];
	int n;
   
	if ((winFd = getfd(gotoWins, MAX_GOTOWIN)) < 0)
		errorDialog("Too Many Windows");
	else
  		{
		mxgoto = rtn = (GotoWin *)malloc(sizeof (GotoWin));
		memset(rtn, '\0', sizeof(*rtn));
		rtn->mainWin = win;
		doFetch(hier, "goto_dialog", toplevel, &rtn->gotoMainWin, &aType);
		gotoWins[winFd] = rtn;
		}
	return rtn;
	}

BookmarkWin * newBookmarkWin(MainWin * win)
	{
	BookmarkWin * rtn;
	MrmType aType;
	int count;
	WinFd winFd;
	Arg args[MAX_ARGS];
	Atom importList[1];
	int n;
   
	if ((winFd = getfd(bookmarkWins, MAX_BOOKMARKWIN)) < 0)
		errorDialog("Too Many Windows");
	else
  		{
		mxbookmark = rtn = (BookmarkWin *)malloc(sizeof(BookmarkWin));
		memset(rtn, '\0', sizeof(*rtn));
		rtn->mainWin = win;
		rtn->bookmarkLastSelected = -1;
		doFetch(hier, "bookmark_dialog", toplevel, &rtn->bookmarkMainWin, &aType);
		XtAddCallback(XtParent(rtn->bookmarkMainWin), XmNpopdownCallback, (XtCallbackProc)bookmark_cancelCB, NULL);
		/* This call causes an Xt error. WHY????????? Not any more! */
		bookmarkWins[winFd] = rtn;
		if (bmarklist == NULL)
			bmarklist = bookmark_create(0);
		showBookmarks(rtn->bookmarkListWidget, bmarklist);
		}
	return rtn;
	}

BookWin * newBookWin(MainWin * win)
	{
	BookWin * rtn;
	MrmType aType;
	int count;
	WinFd winFd;
	Arg args[MAX_ARGS];
	Atom importList[1];
	int n;

	if (win->version == NULL)
		errorDialog(error_noversion);
	else if ((winFd = getfd(bookWins, MAX_BOOKWIN)) < 0)
		errorDialog("Too Many Windows");
	else
  		{
		int num;
		char ** vec;
		XmString head;

		mxbook = rtn = (BookWin *)malloc(sizeof (BookWin));
		memset(rtn, '\0', sizeof(*rtn));
		rtn->mainWin = win;
		doFetch(hier, "book_selection", toplevel, &rtn->bookMainWin, &aType);
		bookWins[winFd] = rtn;
		
		vec = getBookContentsSVec(win->version->title, &num);
		if (vec == NULL)
			errorDialog("Cannot find Contents file");
		else
			{
			populateSelectionFromVec(rtn->bookSelectWidget, vec, num);
			freevector(vec);
			
			head = XmStringCreateSimple(win->version->title);
			XtVaSetValues(rtn->bookMainWin, XmNdialogTitle, head,
						  NULL);
			XmStringFree(head);
/*			XtManageChild(sel);
			setCursor(sel); */
			}
		}
	return rtn;
	}

MotifContext * loadmotif(int argc, char **argv)
	{
	static char *uid_files[] = {"bib.uid"};
	MotifContext * mcon;
/*	static char dragTranslations[] = "#override  <Btn2Down>:  StartDrag()"; */
/*	static char dragTranslations[] = "<Btn2Down>  :  process-bdrag() my-drag-start()"; */
	static char dragTranslations[] = "<Btn2Down>  :  my-drag-start()"; 
	static  XtActionsRec dragActions[] =
/*     { {"StartDrag", (XtActionProc)startDrag} }; */
	{ {"my-drag-start", (XtActionProc)startDrag} };
/*	  {"process-bdrag", (XtActionProc)startDrag} }; */  

	if (getenv("UIDPATH") ==  NULL)
		{
		char value[100];
		cpystr(value, "UIDPATH=./%U:", bhome, "/uid/%U", NULL);
		putenv(value);
		}
	mcon = (MotifContext *)malloc(sizeof (MotifContext));


	toplevel = XtVaAppInitialize(&mcon->appContext, "Bib", NULL, 0, &argc, argv, fallbacks, NULL);
#   ifdef NEVER
	XtToolkitInitialize();
	mcon->appContext = XtCreateApplicationContext();
	if ((display = XtOpenDisplay(mcon->appContext, NULL, argv[0], "Bib", NULL, 0, &argc, argv)) == NULL)
		{
		fprintf(stderr, "Cannot open Display\n");
		exit(1);
		}
	toplevel = XtAppCreateShell(argv[0], "Bib", applicationShellWidgetClass, display, NULL, 0);
#   endif

   /* override button two press to start a drag */
   parsed_xlations = XtParseTranslationTable(dragTranslations);
   XtAppAddActions(mcon->appContext, dragActions,
                     XtNumber(dragActions)); 


	COMPOUND_TEXT = XmInternAtom(XtDisplay(toplevel), "COMPOUND_TEXT", False);
	MrmInitialize();
	if (MrmOpenHierarchyPerDisplay(XtDisplay(toplevel), 1, uid_files, NULL, &hier) != MrmSUCCESS)
		{
		fprintf(stderr, "Cannot open Hierarchy\n");
		return NULL;
		}
	MrmRegisterNames(register_vector, register_vector_length);

	newMainWin();
#ifdef NEVER
	if (MrmFetchWidget(hier, "bib", toplevel, &aWidget, &aType) != MrmSUCCESS)
		{
		fprintf(stderr, "Cannot fetch top Widget\n");
		return NULL;
		}
	XtManageChild(aWidget);
	XtRealizeWidget(toplevel);
#endif
	watchCursor = XCreateFontCursor(XtDisplay(toplevel), XC_watch);
	stdCursor = XCreateFontCursor(XtDisplay(toplevel), XC_top_left_arrow);
	mcon->context.run = (int (*)(Context *))runmotif;
	return mcon;
	}

XmString * vecToXmStringVec(char ** vec, int size)
	{
	XmString * rtn, * sptr;
	rtn = sptr = (XmString *)malloc(sizeof(XmString) * (size + 1));
	for (; *vec != NULL; vec++, sptr++)
		*sptr = XmStringCreateSimple(*vec);
	*sptr = NULL;
	return rtn;
	}

void freeXmStringVec(XmString * vec)
	{
	XmString * ptr;
	ptr = vec;
	while (*ptr != NULL)
		XmStringFree(*ptr++);
	free(vec);
	}

void vecToMenu(Widget w, XtCallbackProc p, char ** vec, int size)
	{
	XmString str;
	Widget button;
	for (; *vec != NULL; vec++)
		{
		str = XmStringCreateSimple(*vec);
		button = XtVaCreateManagedWidget("button", xmPushButtonWidgetClass, w,
										 XmNlabelString, str,
										 NULL);
		XtAddCallback(button, XmNactivateCallback, p, strdup(*vec));
		XmStringFree(str);
/*		XtManageChild(button); */
		}
	}

void registerOpenMenuCB(Widget w,
						XtPointer client_data,
						XmAnyCallbackStruct *cbs)
	{
	char ** vec;
	int num;
	char filename[PATHSIZE+1];

	mxwin->openWidget = w;
	
	vec = getVersionListSVec(&num);
	if (vec == NULL)
		errorDialog("Contents file not found");
	else
		{
		vecToMenu(w, (XtCallbackProc)openVersionCB, vec, num);
		freevector(vec);
		}
	}


void populateSelectionFromVec(Widget w, char ** vec, int num)
{
	FILE * file;
	char buf[500];
	XmString * svec;

	svec = vecToXmStringVec(vec, num);
	XtVaSetValues(w, XmNlistItemCount, num,
				  XmNlistItems, svec,
				  NULL);
	freeXmStringVec(svec);
}

Bool popupWindow(MrmHierarchy hier, char * name, Widget parent, Widget * widget, MrmType * type)
	{
	Bool rtn = YES;
	if (MrmFetchWidget(hier, name, parent, widget, type) != MrmSUCCESS)
		{
		errorDialog("cannot fetch");
		rtn = NO;
		}
	return rtn;
	}


void openVersionCB(Widget w,
			 char * client_data,
			 XmAnyCallbackStruct *cbs)
	{
	MainWin * win;
	if ((win = winOfMain(w)) == NULL)
		fprintf(stderr, "openver: win not found\n");
	win->version = loadVersion(client_data);
	assert(0 < win->version->numBooks && win->version->books);
	loadBookAsCurrent(win->version, win->version->books[0].title);
	showBook(win);
	}

void gotoCB(Widget w,
			 XtPointer client_data,
			 XmAnyCallbackStruct *cbs)
	{
	newGotoWin(winOfMain(w));
/*
	Widget sel;
	MrmType mtype;
	
	doFetch(hier, "goto_dialog", w, &sel, &mtype);
	*/
	}

void bookListCB(Widget w,
				XtPointer client_data,
				XmAnyCallbackStruct *cbs)
	{
	newBookWin(winOfMain(w));
#ifdef NEVER
	MrmType mtype;
	Widget sel;
	
	if (cVersion == NULL)
		errorDialog(error_noversion);
	else if (doFetch(hier, "book_selection", w, &sel, &mtype))
		{
		int num;
		char ** vec;
		XmString head;
		
		vec = getBookContentsSVec(cVersion->title, &num);
		if (vec == NULL)
			errorDialog("Cannot find Contents file");
		else
			{
			populateSelectionFromVec(mxbook->bookSelectWidget, vec, num);
			freevector(vec);
			
			head = XmStringCreateSimple(cVersion->title);
			XtVaSetValues(sel, XmNdialogTitle, head,
						  NULL);
			XmStringFree(head);
			XtManageChild(sel);
			setCursor(sel);
			}
		}
#endif
	}

void bookmark_cancelCB(Widget w, 
					   XtPointer client_data,
					   XmAnyCallbackStruct *cbs) 
	{ 
	BookmarkWin * win = winOfBookmark(w);
	XtDestroyWidget(XtParent(win->bookmarkMainWin));
	XtVaSetValues(win->mainWin->menuBookmarkWidget, XmNsensitive, TRUE, NULL);
	}

void showBookmarks(Widget wList, BookmarkList * blist)
	{
	XmStringTable table;
	int count;
	char buf[1024+1];
	Bookmark * bmark;
	char rbuf[1024+1];
	
	if (blist == NULL)
		XtVaSetValues(wList, XmNitemCount, 0, NULL);
	else
		{
		table = (XmStringTable)malloc(blist->num * sizeof(XmString));
		for (count = 0; count < blist->num; count++)
			{
			bmark = &blist->bookmarks[count];
			sprintf(buf, "%-20s %s", bmark->title, rangeToBuf(bmark->range, rbuf));
			table[count] = XmStringCreateSimple(buf);
			}
		XtVaSetValues(wList, XmNitems, table, XmNitemCount, count, NULL);
		}
	}

void bookmarkCB(Widget w,
				XtPointer client_data,
				XmAnyCallbackStruct *cbs)
	{
	newBookmarkWin(winOfMain(w));
#ifdef NEVER
	if (mxbookmark->bookmarkMainWin == NULL)
		{
		MrmType mtype;
		doFetch(hier, "bookmark_dialog", w, &mxbookmark->bookmarkMainWin, &mtype);
		XtAddCallback(XtParent(mxbookmark->bookmarkMainWin), XmNpopdownCallback, (XtCallbackProc)bookmark_cancelCB, NULL);
		/* This call causes an Xt error. WHY????????? Not any more! */
		}
	if (mxbookmark->bookmarkMainWin != NULL)
		{
		XtVaSetValues(mxbookmark->menuBookmarkWidget, XmNsensitive, FALSE, NULL);
		if (bmarklist == NULL)
			bmarklist = bookmark_create(0);
		showBookmarks(mxbookmark->bookmarkListWidget, bmarklist);
		}
#endif
	}

void searchCB(Widget w,
			  XtPointer client_data,
			  XmAnyCallbackStruct *cbs)
	{
/*
	Widget sel;
	MrmType mtype;
	doFetch(hier, "search_dialog", w, &sel, &mtype);
	*/
	newSearchWin(winOfMain(w));
	}

void bookCancelCB(Widget w,
			  XtPointer client_data,
			  XmSelectionBoxCallbackStruct *cbs)
	{
	XtDestroyWidget(XtParent(w));
	}

void bookApplyCB(Widget w,
			  XtPointer client_data,
			  XmSelectionBoxCallbackStruct *cbs)
	{
	char *value;
	char *bookContents, *headvalue;
	Book * book;
	XmString head;
	struct stat sstat;
	char filename[PATHSIZE];
	int fd;
	BookWin * win = winOfBook(w);
	
	XtVaGetValues(XtParent(w), XmNdialogTitle, &head, NULL);
	XmStringGetLtoR(head, XmSTRING_DEFAULT_CHARSET, &headvalue);
	XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &value);
	win->mainWin->version = loadBookVersion(headvalue, value);
	showBook(win->mainWin);
	}

void bookOkCB(Widget w,
			 XtPointer client_data,
				 XmSelectionBoxCallbackStruct *cbs)
	{
	bookApplyCB(w, client_data, cbs);
	XtDestroyWidget(XtParent(XtParent(w)));
	}

void exitCB(Widget w,
			XtPointer client_data,
			XmSelectionBoxCallbackStruct *cbs)
	{
fprintf(stderr, "exit\n");
	exit(0);
	}

void newCB(Widget w,
			  XtPointer client_data,
			  XmAnyCallbackStruct *cbs)
	{
	MainWin * win;
	win = newMainWin();
/*
	Widget aWidget;
	MrmType aType;

	fprintf(stderr, "newCB\n");
	if (MrmFetchWidget(hier, "bib", toplevel, &aWidget, &aType) != MrmSUCCESS)
		{
		fprintf(stderr, "Cannot fetch top Widget\n");
		}
	XtManageChild(aWidget);
	XtRealizeWidget(toplevel);
	*/
	}

void saveAsCB(Widget w,
			  XtPointer client_data,
			  XmAnyCallbackStruct *cbs)
	{
	Widget sel;
	MrmType mtype;
	doFetch(hier, "save_as", w, &sel, &mtype);
	}

void printerCB(Widget w,
			   XtPointer client_data,
			   XmAnyCallbackStruct *cbs)
	{
	Widget sel;
	MrmType mtype;
	doFetch(hier, "printer_dialog", w, &sel, &mtype);
	}

static void showVerse(MainWin * win, Book * book, AbsVerse ver)
{
	message(win, NULL);
	XmTextSetCursorPosition(win->textWidget, book->verses[ver]);
	XmTextSetCursorPosition(win->textWidget, endOfVerse(book, ver));
}

char * searching = "Searching...";

void searchForward(MainWin * win, Pos pos, char * expression, char * srange)
{
	int ver;
	Range * range;
	ErrorBuf errbuf;
	Bool nBook;
	int eaten;

	if (win->version == NULL)
		errorDialog(error_noversion);
	else
	{
		message(win, searching);
/*		busy(earch->searchMainWin); */
		range = parseVerse(srange, &eaten);
		ver = nextSearchVerse(win->version->cBook, pos);
		ver = searchNextVersion(win->version, ver, range, expression, errbuf, NO, &nBook);
		checkShowVerse(win, ver, nBook);
/*		notbusy(mxsearch->searchMainWin); */
	}
}

void searchPrev(MainWin * win, Pos pos, char * expression, char * srange)
{
	AbsVerse ver;
	Range * range;
	ErrorBuf errbuf;
	Bool nBook;
	int eaten;

	if (win->version == NULL)
		errorDialog(error_noversion);
	else
	{
		message(win, searching);
/*		busy(mxsearch->searchMainWin); */
		range = parseVerse(srange, &eaten);
		ver = prevSearchVerse(win->version->cBook, pos);
		ver = searchNextVersion(win->version, ver, range, expression, errbuf, YES, &nBook);
		checkShowVerse(win, ver, nBook);
/*		notbusy(mxsearch->searchMainWin); */
	}
}

void search_cancelCB(Widget w,
				XtPointer client_data,
				XmAnyCallbackStruct *cbs)
	{
	SearchWin * win = winOfSearch(w);
	
	XtDestroyWidget(XtParent(win->searchMainWin));
	}

void search_helpCB(Widget w,
			  XtPointer client_data,
			  XmAnyCallbackStruct *cbs)
	{
	}

char * getSearchExp(SearchWin * win)
	{
	return XmTextGetString(XmCommandGetChild(win->searchFor, XmDIALOG_COMMAND_TEXT));
	}

char * getRangeExp(SearchWin * win)
	{
	return XmTextGetString(XmCommandGetChild(win->range, XmDIALOG_COMMAND_TEXT));
	}

void doSearchFirst(MainWin * win, char * expression, char * srange)
	{
	if (win->version == NULL)
		errorDialog(error_noversion);
	else
		{
		Bool searchAll = YES;
		if (searchAll)
			{
			loadBookAsCurrent(win->version, win->version->books[0].title);
			showBook(win);
			}
		searchForward(win, 0, expression, srange);
		}
	}


void search_firstCB(Widget w,
					XtPointer client_data,
					XmAnyCallbackStruct *cbs)
	{
	SearchWin * win = winOfSearch(w);
	busy(win->searchMainWin);
	doSearchFirst(win->mainWin, getSearchExp(win), getRangeExp(win));
	notbusy(win->searchMainWin);
	}

void doSearchLast(MainWin * win, char * expression, char * srange)
	{
	if (win->version == NULL)
		errorDialog(error_noversion);
	else
		{
		Bool searchAll = YES;
		if (searchAll)
			{
			loadBookAsCurrent(win->version, win->version->books[win->version->numBooks-1].title);
			showBook(win);
			}
		searchPrev(win, win->version->cBook->size - 1, expression, srange);
		}
	}

void search_lastCB(Widget w,
				   XtPointer client_data,
				   XmAnyCallbackStruct *cbs)
	{
	SearchWin * win = winOfSearch(w);
	busy(win->searchMainWin);
	doSearchLast(win->mainWin, getSearchExp(win), getRangeExp(win));
	notbusy(win->searchMainWin);
	}

void doSearchNext(MainWin * win, char * expression, char * srange)
	{
	searchForward(win, XmTextGetCursorPosition(win->textWidget), expression, srange);
	}

void search_nextCB(Widget w,
				   XtPointer client_data,
				   XmAnyCallbackStruct *cbs)
	{
	SearchWin * win = winOfSearch(w);
	doSearchNext(win->mainWin, getSearchExp(win), getRangeExp(win));
	}

void doSearchPrev(MainWin * win, char * expression, char * srange)
	{
	searchPrev(win, XmTextGetCursorPosition(win->textWidget), expression, srange);
	}

void search_prevCB(Widget w,
				   XtPointer client_data,
				   XmAnyCallbackStruct *cbs)
	{
	SearchWin * win = winOfSearch(w);
	doSearchPrev(win->mainWin, getSearchExp(win), getRangeExp(win));
	}


void checkShowVerse(MainWin * win, AbsVerse ver, Bool nBook)
	{
	if (ver == noVerseNum)
		message(win, "invalid verse");
	else
		{
		message(win, NULL);
		if (nBook)
			showBook(win);
		showVerse(win, win->version->cBook, ver);
		}
	}

void doGoto(MainWin * win, char * verse) 
	{
	if (win->version == NULL)
		errorDialog(error_noversion);
	else
		{
		AbsVerse ver;
		Bool nBook;
		
		ver = gotoVerseString(win->version, verse, &nBook);
		checkShowVerse(win, ver, nBook);
		}
	}

void goto_okCB(Widget w,
			  XtPointer client_data,
			  XmAnyCallbackStruct *cbs)
	{
	AbsVerse ver;
	Bool nBook;
	char * verse;
	GotoWin * win = winOfGoto(w);
	
	verse = XmTextGetString(XmCommandGetChild(win->gotoCommandWidget, XmDIALOG_COMMAND_TEXT));		
	busy(win->gotoMainWin);
	doGoto(win->mainWin, verse);
	notbusy(win->gotoMainWin);
	}
void goto_cancelCB(Widget w,
				   XtPointer client_data,
				   XmAnyCallbackStruct *cbs)
	
	{
	GotoWin * win = winOfGoto(w);
	
	XtDestroyWidget(XtParent(win->gotoMainWin));
	}

void goto_helpCB(Widget w,
				 XtPointer client_data,
				 XmAnyCallbackStruct *cbs)
	{
	}

void fileselectCB(Widget w,
					 void (*client_data)(Widget w, char *),
				  XmFileSelectionBoxCallbackStruct *cbs)
	{
	char * filename;
	if (!XmStringGetLtoR(cbs->value, XmSTRING_DEFAULT_CHARSET, &filename))
		return;
	client_data(w, filename);
	XtFree(filename);
	XtUnmanageChild(w);
	}

void filecancelCB(Widget w,
				  void (*client_data)(char *),
				  XmFileSelectionBoxCallbackStruct *cbs)
	{
	XtUnmanageChild(w);
	}

Bool prompt_filename(Widget w, void (*proc)(Widget w, char *))
	{
	if (w == NULL)
		{
		w = XmCreateFileSelectionDialog(toplevel, "fileselect", NULL, 0);
		XtAddCallback(w, XmNokCallback, (XtCallbackProc)fileselectCB, proc);
		XtAddCallback(w, XmNcancelCallback, (XtCallbackProc)filecancelCB, proc);
		}
	XtManageChild(w);
	}


void bookmark_open(Widget w, char * filename)
	{
	BookmarkWin * win = winOfBookmark(w);

	bmarklist = reopenBookmark(filename, bmarklist);
	showBookmarks(win->bookmarkListWidget, bmarklist);
	}

void bookmark_load(Widget w, char * filename)
	{
	BookmarkWin * win = winOfBookmark(w);

	bmarklist = combinedLoadBookmark(filename, bmarklist);
	showBookmarks(win->bookmarkListWidget, bmarklist);
	}

void bookmark_openCB(Widget w,
					 XtPointer client_data,
					 XmAnyCallbackStruct *cbs)
	{
	BookmarkWin * win = winOfBookmark(w);
	
	prompt_filename(win->bookmarkFileWidget, bookmark_open);
	}
void bookmark_loadCB(Widget w,
					 XtPointer client_data,
					 XmAnyCallbackStruct *cbs)
	{
	BookmarkWin * win = winOfBookmark(w);

	prompt_filename(win->bookmarkFileWidget, bookmark_load);
	}
void bookmark_saveCB(Widget w,
					 XtPointer client_data,
					 XmAnyCallbackStruct *cbs)
	{
	}
void bookmark_saveasCB(Widget w,
					   XtPointer client_data,
					   XmAnyCallbackStruct *cbs)
	{
	BookmarkWin * win = winOfBookmark(w);

	}

void bookmark_clearCB(Widget w,
					  XtPointer client_data,
					  XmAnyCallbackStruct *cbs)
	{
	BookmarkWin * win = winOfBookmark(w);

	freeBookmarks(bmarklist);
	bmarklist = bookmark_create(0);
	showBookmarks(win->bookmarkListWidget, bmarklist);
	}


void bookmark_gotoCB(Widget w,
					 XtPointer client_data,
					   XmListCallbackStruct *cbs)
	{
	AbsVerse ver;
	Bool nBook;
	Bookmark * mk;
	Verse * verse;
	BookmarkWin * win = winOfBookmark(w);
	
	if (win->mainWin->version == NULL)
		errorDialog(error_noversion);
	else if (0 <= win->bookmarkLastSelected)
		{
		Verse vse;
		mk = &bmarklist->bookmarks[win->bookmarkLastSelected];
		verse = &mk->range->items[0].from;
		
		memcpy(&vse, verse, sizeof(vse));
		normaliseFromVerse(&vse);
		ver = gotoVerse(win->mainWin->version, vse.title, vse.chap, vse.verse, &nBook);
		checkShowVerse(win->mainWin, ver, nBook);
	}
	else
		bookmarkMessage(win, "Select a bookmark first");
}

void bookmark_printCB(Widget w,
					   XtPointer client_data,
					   XmAnyCallbackStruct *cbs)
{
	
}

void bookmark_addCB(Widget w,
					   XtPointer client_data,
					   XmAnyCallbackStruct *cbs)
	{
	Bookmark bmark;
	char * nm, * rng;
	int eaten;
	BookmarkWin * win = winOfBookmark(w);
	
	XtVaGetValues(win->bookmarkNameEntryWidget, XmNvalue, &nm, NULL);
	XtVaGetValues(win->bookmarkVerseEntryWidget, XmNvalue, &rng, NULL);
	bmark.title = strdup((char *)nm);
	bmark.range = parseVerse(rng, &eaten);
	bookmarkAdd(bmarklist, &bmark, win->bookmarkLastSelected);
	showBookmarks(win->bookmarkListWidget, bmarklist);
#ifdef NEVER
	XtVaSetValues(bookmarkMainWin, XmNtopItemPosition, bmarklist->num, NULL);
#endif // NEVER
	}

void bookmark_updateCB(Widget w,
					   XtPointer client_data,
					   XmAnyCallbackStruct *cbs)
{
	Bookmark bmark;
	char * nm, * rng;
	int eaten;
	BookmarkWin * win = winOfBookmark(w);
	
	XtVaGetValues(win->bookmarkNameEntryWidget, XmNvalue, &nm, NULL);
	XtVaGetValues(win->bookmarkVerseEntryWidget, XmNvalue, &rng, NULL);
	bmark.title = strdup((char *)nm);
	bmark.range = parseVerse(rng, &eaten);   
	bookmarkUpdate(bmarklist, &bmark, win->bookmarkLastSelected);
	showBookmarks(win->bookmarkListWidget, bmarklist);
	}

void bookmark_deleteCB(Widget w,
					   XtPointer client_data,
					   XmAnyCallbackStruct *cbs)
	{
	BookmarkWin * win = winOfBookmark(w);

	bookmarkDelete(bmarklist, win->bookmarkLastSelected);
	showBookmarks(win->bookmarkListWidget, bmarklist);
	}

void bookmark_positionCB(Widget w,
					   XtPointer client_data,
					   XmListCallbackStruct *cbs)
	{
	Bookmark * mk;
	char buf[RANGEBUF_SIZE+1];
	BookmarkWin * win = winOfBookmark(w);
	
	win->bookmarkLastSelected = cbs->item_position - 1;
	mk = &bmarklist->bookmarks[win->bookmarkLastSelected];
	XtVaSetValues(win->bookmarkNameEntryWidget, XmNvalue, mk->title, XmNcursorPosition, 0, NULL);
	XtVaSetValues(win->bookmarkVerseEntryWidget, XmNvalue, rangeToBuf(mk->range, buf), XmNcursorPosition, 0, NULL);
	
	}

void bookmark_selectCB(Widget w,
					   XtPointer client_data,
					   XmListCallbackStruct *cbs)
	{
	BookmarkWin * win = winOfBookmark(w);

	win->bookmarkLastSelected = cbs->item_position - 1;
	}

void bookmark_popdownCB(Widget w,
					   XtPointer client_data,
						XmAnyCallbackStruct *cbs)
	{
	BookmarkWin * win = winOfBookmark(w);
	XtVaSetValues(win->mainWin->menuBookmarkWidget, XmNsensitive, TRUE, NULL);
	}

char * getToAt(register char * p, register char * to)
	{
	while (!(*p == EOS || *p == '@'))
		*to++ = *p++;
	*to = EOS;
	return p;
	}



void transferProc(Widget w, 
				  XtPointer closure, 
				  Atom * seltype, 
				  Atom * type, 
				  XtPointer value,
				  unsigned long * length,  
				  int format)  
	{
	int n;
	Arg args[MAX_ARGS];
	char * ascii;
	MainWin * win;
	
	/* information from the drag initiator is passed in
	 * compound compound text format.  Convert it to compound
	 * string and replace the  Label label. */
	
	if (*type == COMPOUND_TEXT) 
		{ 
		XmStringGetLtoR(XmCvtCTToXmString(value), XmSTRING_DEFAULT_CHARSET, &ascii);
		win = winOfMain(closure);
		if (ascii[0] == '?')
			{
			char searchType[100];
			char searchExp[100];
			char rangeExp[100];
			
			ascii = getToAt(ascii + 1, searchType);
			if (*ascii != EOS)
				ascii = getToAt(ascii + 1, searchExp);
			if (*ascii != EOS)
				ascii = getToAt(ascii + 1, rangeExp);
			if (searchType[0] == 'N')
				doSearchNext(win, searchExp, rangeExp);
			else if (searchType[0] == 'P')
				doSearchPrev(win, searchExp, rangeExp);
			else if (searchType[0] == 'F')
				doSearchFirst(win, searchExp, rangeExp);
			else if (searchType[0] == 'L')
				doSearchLast(win, searchExp, rangeExp);
			else
				message(win, "unknown search expression");
			}
		else
			doGoto(win, ascii);
		}
	}

void handleDrop(Widget w, XtPointer client_data, XtPointer call_data)
	{
	XmDropProcCallback      DropData;
	XmDropTransferEntryRec  transferEntries[2];
	XmDropTransferEntry     transferList;
	Arg                     args[MAX_ARGS];
	int                     n;

	DropData = (XmDropProcCallback)call_data;
	
	/* set the transfer resources */
	n = 0;
	
	/* if the action is not Drop or the operation is not Copy,
	 * cancel the drop */
	if ((DropData->dropAction != XmDROP) ||
		(DropData->operation != XmDROP_COPY))
		{
		XtSetArg(args[n], XmNtransferStatus, XmTRANSFER_FAILURE);
		n++;
		}
	else 
		{
		char * expression;
		char * srange;
		char tmpstring[100];
		char       *ctext;
		XmString    cstring;
		char        *passtext;
#ifdef NEVER
		SearchWin * win = winOfSearch(w);
		
		expression = getSearchExp(win);
		srange = getRangeExp(win);
		sprintf(tmpstring, "?%s@%s", expression, srange);
		cstring = XmStringCreateLocalized(tmpstring);
		ctext = XmCvtXmStringToCT(cstring);
		
		passtext = XtMalloc(strlen(ctext)+1);
		memcpy(passtext, ctext, strlen(ctext)+1);
#endif
		


		/* the drop can continue.  Establish the transfer list
		 * and start the transfer */
		transferEntries[0].target = COMPOUND_TEXT;
		transferEntries[0].client_data = (XtPointer)w;
		transferList = transferEntries; 
		XtSetArg(args[n], XmNdropTransfers, transferList); n++;
		XtSetArg(args[n], XmNnumDropTransfers, 1); n++;
		XtSetArg(args[n], XmNtransferProc, transferProc); n++; 
		}
	
	/* start the transfer or cancel */
	XmDropTransferStart(DropData->dragContext, args, n); 
	}


XtPointer oldFunc;

static Boolean dragConvertProc(Widget w, 
							   Atom * selection, 
							   Atom * target, 
							   Atom * typeRtn,
							   XtPointer * valueRtn, 
							   unsigned long *lengthRtn, 
							   int * formatRtn,
							   unsigned long * max_lengthRtn, 
							   XtPointer client_data,
							   XtRequestId * request_id) 
	{
	Widget      dc;
	XmString    cstring;
	char * searchType = NULL;
	char tmpstring[100];
	int         *value;
	int         n;
	Arg         args[MAX_ARGS];
	char       *ctext;
	char        *passtext;
	char * expression;
	char * srange;
	SearchWin * win;
	

	/* this routine processes only compound text */
	if (*target != COMPOUND_TEXT)
		return(False);
	XtVaGetValues(w, XmNclientData, &win, NULL);
/*	if ((win = winOfSearch(w)) == NULL) */
	if (win == NULL)
		fprintf(stderr, "cannot find search win\n"); 
	expression = getSearchExp(win);
	srange = getRangeExp(win);
	if (win->drag == win->next)
		searchType = "N";
	else if (win->drag == win->prev)
		searchType = "P";
	else if (win->drag == win->first)
		searchType = "F";
	else if (win->drag == win->last)
		searchType = "L";
	else
		fprintf(stderr, "unknown search type\n");
	
	sprintf(tmpstring, "?%s@%s@%s", searchType, expression, srange);
fprintf(stderr, "dragConvertProc: '%s'\n", tmpstring);	
	cstring = XmStringCreateLocalized(tmpstring);
	ctext = XmCvtXmStringToCT(cstring);
	
	passtext = XtMalloc(strlen(ctext)+1);
	memcpy(passtext, ctext, strlen(ctext)+1);
	
	/* format the value for transfer.  convert the value from
	 * compound string to compound text for the transfer */
	*typeRtn = COMPOUND_TEXT;
	*valueRtn = (XtPointer) passtext;
	*lengthRtn = strlen(passtext);
	*formatRtn = 8;
	return(True); 
	}

/* This routine is performed by the initiator when a drag
 * starts  (in this case, when mouse button 2 was pressed).
 * It starts  the drag processing, and establishes a drag
 * context. */


void startDrag(Widget w, XEvent * event)
	{
	Arg             args[MAX_ARGS];
	Cardinal        n;
	Atom            exportList[1];
	Widget dragContext;
	SearchWin * win;

	fprintf(stderr, "startDrag\n");	
	if ((win = winOfSearch(w)) == NULL)
		fprintf(stderr, "cannot find search win xx\n");
	win->drag = w;
	n = 0;
	XtSetArg(args[n], XmNexportTargets, exportList); n++;
	XtSetArg(args[n], XmNnumExportTargets, 1); n++;
	XtSetArg(args[n], XmNdragOperations, XmDROP_COPY); n++;
	XtSetArg(args[n], XmNconvertProc, dragConvertProc); n++;
	XtSetArg(args[n], XmNclientData, win); n++;
/*	XtSetValues(dragContext, args, n); */
	XmDragStart(w, event, args, n);  
	}

/*
extern Widget XmGetDragContext(
                        Widget w,
                        Time time) ; */
